/*
 *	3DShell
 *	3DShell.cpp - Main engine setup, Win32 event handling & passing of mouse clicks/movement to cScene.
 *	Iain C, DATEMARK
*/

// ********************************* INCLUDES **********************************************

#include "GeneralHeader.h"			// Includes everything we need

// ********************************* GLOBALS **********************************************

HINSTANCE	hInstance;				// Holds The Instance Of The Application
HDC			hDC		= NULL;			// Private GDI Device Context
HGLRC		hRC		= NULL;			// Permanent Rendering Context
HWND		hWnd	= NULL;			// Holds Our Window Handle
bool		active	= TRUE;			// Window Active Flag Set To TRUE By Default
bool		left	= false;		// Left button currently down?
bool		right	= false;		// Right button currently down?
int			omx, omy, mx, my;		// Current & old mouse co-ords
bool		keys[256];				// Array used for key handling

int			windowCount;			// Window count (for the taskmanager)
char		windowTitle[200][256];	// Arrays of window info (for the taskmanager)
bool		windowMinimised[200];
HWND		windowHWnd[200];

UINT		HookMessage;			// The message used by the hook to communicate us what is going on in the system.

CScene		cScene;					// The scene object

// ********************************* CALL-ME-BACKS **********************************************

LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc
BOOL	CALLBACK EnumWindowsProc(HWND, LPARAM);			// Declaration for EnumWindowsProc

// ********************************* SETUP WINDOW HOOKS **********************************************

int SetupWindowHooks()
{
	HookMessage = RegisterWindowMessage("HookMessage");
	InitHooksDll(hWnd, HookMessage);
	InstallFilters();
	return TRUE;
}

// ********************************* RELEASE WINDOW HOOKS **********************************************

int ReleaseWindowHooks()
{
	UnInstallFilters();
	ShutdownHooksDll();
	return TRUE;
}

// ********************************* BUILD WINDOW LIST **********************************************

int BuildWindowList()
{
	windowCount=0;
	EnumWindows((WNDENUMPROC)EnumWindowsProc, 0);
//	cScene.cTaskMan.DisplayWindows(*windowTitle, window
	return TRUE;
}

// ********************************* ENUMWINDOWS CALLBACK **********************************************

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	char buffer[256]; 
	
	if ((GetWindowLong(hwnd,GWL_HWNDPARENT)==0) && (IsWindowVisible(hwnd) || IsIconic(hwnd))) // Make sure this is a visible, or iconized, parent window. 
	{ 
		GetWindowText(hwnd, buffer, 256);		// Get the window title.

		if (strlen(buffer))  // Only add non-blank titles. This will prevent invisible windows from showing up. Also eliminates pop-ups. 
		{
			windowHWnd[windowCount] = hwnd;
			sprintf(windowTitle[windowCount], buffer);
			if (IsIconic(hwnd))
			{
				windowMinimised[windowCount] = true;
			} else {
				windowMinimised[windowCount] = false;
			}
			windowCount++;
		} 
	}

	return TRUE;	// Continue enumerating
}

// ********************************* SEND WINDOW TO BACK **********************************************

void SendToBack()						// Make us bottom of z-order to fake a desktop
{
	SetWindowPos(hWnd, HWND_BOTTOM, -3, -19, GetSystemMetrics(SM_CXSCREEN)+6, GetSystemMetrics(SM_CYSCREEN)+30, 0);
}

// ********************************* TOGGLE TASKBAR **********************************************

void ToggleTaskbar()
{
	HWND taskbar = NULL;
	taskbar = FindWindow("Shell_TrayWnd", "");
	if (IsWindowVisible(taskbar))
	{
		ShowWindow(taskbar, SW_HIDE);
	} else {
		ShowWindow(taskbar, SW_SHOW);
	}
}

// ********************************* SHOW TASKBAR **********************************************

void ShowTaskbar()
{
	HWND taskbar = NULL;
	taskbar = FindWindow("Shell_TrayWnd", "");
	ShowWindow(taskbar, SW_SHOW);
}

// ********************************* KILL THE WINDOW **********************************************

void KillGLWindow()									// Properly Kill The Window
{
	if (hRC)											// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Set hWnd To NULL
	}

	if (!UnregisterClass("OpenGL",hInstance))			// Are We Able To Unregister Class
	{
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;									// Set hInstance To NULL
	}
}

// ********************************* GO TO OPEN GL **********************************************

int CreateGLWindow()
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style
	RECT		WindowRect;				// Grabs Rectangle Upper Left / Lower Right Values
	WindowRect.left=(long)0;			// Set Left Value To 0
	WindowRect.right=(long)1;			// Set Right Value To Width (temporary only - we call Resize right after creating it & it gets its proper size then.)
	WindowRect.top=(long)0;				// Set Top Value To 0
	WindowRect.bottom=(long)1;			// Set Bottom Value To Height

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Icon
	wc.hCursor			= LoadCursor(NULL, IDC_CROSS);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	dwExStyle=WS_EX_TOOLWINDOW;
	dwStyle=0;

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);		// Adjust Window To True Requested Size

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,							// Extended Style For The Window
								"OpenGL",							// Class Name
								WINDOW_TITLE,						// Window Title
								dwStyle |							// Defined Window Style
								WS_CLIPSIBLINGS |					// Required Window Style
								WS_CLIPCHILDREN,					// Required Window Style
								0, 0,								// Window Position
								WindowRect.right-WindowRect.left,	// Calculate Window Width
								WindowRect.bottom-WindowRect.top,	// Calculate Window Height
								NULL,								// No Parent Window
								NULL,								// No Menu
								hInstance,							// Instance
								NULL)))								// Dont Pass Anything To WM_CREATE
/*	if (!(hWnd=GetDesktopWindow()))*/
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		RES_BITDEPTH,								// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};

	if (!(hDC=GetDC(hWnd)))								// Did We Get A Device Context?*/
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetFocus(hWnd);									// Sets KeyScene Focus To The Window
	SendToBack();									// Fake a desktop
	//HideTaskbar();									// Hide the taskbar
	cScene.ReSizeGLScene(GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN)); // Set Up Our Perspective GL Screen

	if (!cScene.PostGLInit())					// Initialize the scene
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

// ********************************* HANDLE WIN32 EVENTS **********************************************

LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
				SendToBack();						// Send to back
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// We're having none of that
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Let CScene Resize The OpenGL Window
		{
			cScene.ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}

		case WM_LBUTTONDOWN:						//Check for left mouse button.
		{
			mx = LOWORD(lParam);
			my = HIWORD(lParam);
			left = true;
			cScene.FindHit(mx, my); //In case of a click, let CScene check for hits on its objects
			return 0;
		}

		case WM_LBUTTONUP:
		{
			left = false;
			return 0;
		}
		
		case WM_RBUTTONDOWN:
		{
			cScene.PauseRotation();
			SetCapture(hWnd);			// If we don't set the capture we won't get mouse move messages when the mouse moves outside the window. Although it probably won't anyway cos of the way it's all happening.
			mx = LOWORD(lParam);
			my = HIWORD(lParam);
			right = true;
			return 0;
		}

		case WM_RBUTTONUP:
		{
			cScene.ContinueRotation();
			ReleaseCapture();	// remember to release the capture when we are finished
			right = false;
			return 0;
		}

		case WM_MOUSEMOVE:
		{
			if (left || right)	// There's a button down, worth our while processing
			{
				omx = mx;	// put old co-ords in here...
				omy = my;
				mx = LOWORD(lParam);	// ...and new ones in here
				my = HIWORD(lParam);
				/* Win32 is pretty braindead about the x, y position that
				   it returns when the mouse is off the left or top edge
				   of the window (due to them being unsigned). therefore,
				   roll the Win32's 0..2^16 pointer co-ord range to the
				   more amenable (and useful) 0..+/-2^15. */
				if(mx & 1 << 15) mx -= (1 << 16);
				if(my & 1 << 15) my -= (1 << 16);
				//if (left);
				if (right) cScene.ProcessMouseMovement(mx, my, omx, omy);	// Let CScene process movement
			}
			return 0;
		}
		//return 0;
	}

	if(uMsg == HookMessage)	// Check for our window hook message from the DLL
	{
		switch(wParam)
		{
			case MD_CBT_CREATE:
			{
				return 0;
			}
			case MD_CBT_DESTROY:
			{
				return 0;
			}
			case MD_SH_CREATED:
			{
				//The hook doesn't send us windows we don't need; the exception would be the shell itself, for this we need to check.
				if(hWnd != (HWND)lParam)	// It's not our hWnd
				{
					cScene.cTaskMan.AddTask ((HWND)lParam);	// Add the task if it's not already there
				}
				return 0;
			}
			case MD_SH_DESTROYED:
			{
				cScene.cTaskMan.RemoveTask ((HWND)lParam);	// Remove the task
				return 0;
			}
			case MD_SH_ACTIVATED:
			{
				/*
				//Check if the window is an MS-DOS window, it's an exception, 
				//we don't get notified of their creation.

				//Check the class name of the hWnd				
				TCHAR dosWndClassName[] = "ConsoleWindowClass";
				TCHAR wndClassName[256];

				::GetClassName((HWND)lParam, wndClassName, 256);

				if(lstrcmpi(dosWndClassName, wndClassName) == 0)
				{
					//If the window exists in the current desktop, it's already been handled.
					if(!m_pData->Exists((HWND)lParam))
					{
						//It's the first time the dos box is activated add it to our list.
						m_pData->AddWindow((HWND)lParam);
					}
				}
				*/
				//The hook doesn't send us windows we don't need; the exception would be the shell itself, for this we need to check.
				if(hWnd != (HWND)lParam)
				{
					cScene.cTaskMan.UpdateTask ((HWND)lParam);	// Add the task if it's not already there, else update it
				}				
				return 0;
			}
		}
	}

	// Pass All Unhandled Messages To DefWindowProc	
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

// ********************************* ENTRY POINT **********************************************

int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop

	// Create Our OpenGL Window
	if (!CreateGLWindow())
	{
		return 0;									// Quit If Window Was Not Created
	}

	// Set up window hooks
	SetupWindowHooks();

	while(!done)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
		else										// If There Are No Messages
		{
			if (keys[VK_F9]){
				keys[VK_F9]=false;
				BuildWindowList();		// Force an update
			}

			if (keys[VK_F7]){
				keys[VK_F7]=false;
				cScene.ToggleRotation();
			}

			if (keys[VK_F1]){
				keys[VK_F1]=false;
				ToggleTaskbar();
			}

			if (keys[VK_F2]){
				keys[VK_F2]=false;
				cScene.SetCamTrackPath(CAMPATH_TRACK_TO_TASKMAN);
			}

			if (keys[VK_F3]){
				keys[VK_F3]=false;
				cScene.SetCamTrackPath(CAMPATH_TRACK_TO_WHARF);
			}

			// Draw The Scene.  Watch For F8 Key And Quit Messages From DrawGLScene()
			if ((active && !cScene.Draw()) || keys[VK_F8])	// Active?  Was There A Quit Received?
			{
				done=TRUE;							// ESC or DrawGLScene Signalled A Quit
			}
			else									// Not Time To Quit, Update Screen
			{
				SwapBuffers(hDC);					// Swap Buffers (Double Buffering)
			}
		}
	}

	// Shutdown
	ReleaseWindowHooks();							// release window hooks
	KillGLWindow();									// Kill The Window
	return (msg.wParam);							// Exit The Program
}
