/*
 *	3DShell
 *	Scene.h - Header file for class CScene - the scene graph
 *	Iain C, DATEMARK
 */

class CScene
{
public:
	// Constructor & destructor
	CScene();																// Setup default values
	virtual ~CScene();

	// Functions
	int				Draw();													// Draw	the Scene: Handles the camera too. Actually it's just the Scene we move, the camera always renders at (0,0,0).
	int				FindHit(int mx, int my);								// Find mouse hit
	int				PostGLInit();											// Do Post-GL setup; set gl config, load world
	int				ProcessMouseMovement(int mx, int my, int omx, int omy);	// Handle mouse events
	int				ReSizeGLScene(GLsizei width, GLsizei height);			// Resize And Initialize The GL Window
	
	int				PauseRotation();										// Rotation on/off
	int				ContinueRotation();
	int				ToggleRotation();
	int				SetCamTrackPath(int PathToTrack);						// Track the cam to a pre-defined position

	// Properties
	CWharf			cWharf;													// The wharf object
	CTaskManager	cTaskMan;												// The task manager object
	CBackdrop		cBackdrop;												// The backdrop/skybox object

	bool			LockoutControls;										// Flag to lock out controls

private:
	// Functions
	int				Spin(float moveAmount);									// Spin the Scene CW/CCW
	int				Tilt(float moveAmount);									// Tilt the Scene forwards/backwards
	int				DrawSceneMembers();										// Loop through scene members drawing them: Called by Draw & FindHit
	int				SetupSceneMembers();									// Setup world objects & init them
	int				ProcessHit(int objectName);								// Process a hit on specified object
	int				GetNewObjectGroupName(char *HumanReadableGroupName);	// Get new base object group name for specified group
	bool			ObjectIsInGroup(int ObjectName, char *HumanReadableGroupName);	// Returns true if object is in group, false if not
	int				DoCamTrack();											// Do a track frame

	// Properties
	GLfloat			SceneRotX, SceneRotZ, SceneDistance;					// Scene positioning
	GLfloat			LightAmbient[4];										// Ambient light
	GLfloat			LightDiffuse[4];										// Diffuse light
	GLfloat			LightPosition[4];										// Light position
	GLfloat			AspectRatio;											// Aspect ratio of viewport
	GLfloat			TrackingPosX[CAMPATH_TRACK_OVER_FRAMES];
	GLfloat			TrackingPosZ[CAMPATH_TRACK_OVER_FRAMES];
	int				NumberGroups;											// Number of group in scene
	char			GroupNameLookupHRGN[500][50];							// To lookup HRGNs/GNs
	int				GroupNameLookupGN[50];
	bool			AutoRotate;												// Automatic rotation flag
	bool			WasRotating;											// Backup flag for AutoRotate on pause/resume
	bool			IsTracking;												// Camera's on a track so we need to call DoCamTrack each scene.		
	int				TrackFrame;												// Current frame of tracking sequence
};