/*
 *	3DShell
 *	TaskManager.cpp - Implements the TaskManager
 *	Iain C, DATEMARK
 */

// ********************************* INCLUDES **********************************************

#include "GeneralHeader.h"

// ********************************* CONSTRUCTORS/DESTRUCTORS **********************************************

CTaskManager::CTaskManager()
{
	start = NULL;
	TaskCount = 0;
}

CTaskManager::~CTaskManager()
{
	nodeptr p = start, n;
	while (p != NULL)
	{
		n = p;
		p = p->next;
		delete n;
	}
}

// ********************************* METHODS **********************************************

int CTaskManager::PostGLInit(int m_NamesOffset)	// Do Post-GL init, set base name
{
	NamesOffset = m_NamesOffset;
	return TRUE;
}

int CTaskManager::Draw()		// Draw tasks in our group
{
	nodeptr p = start;
	float RotatePosition = 0;
	while (p != NULL)
	{
		glPushMatrix();
			glRotatef(RotatePosition,0.0f,0.0f,5.0f);
			glTranslatef(0.0f,0.0f,5.0f);	// Translate from world origin
			glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
			glTranslatef(0.0f,0.0f,20.0f);	
			p->Draw();
			p = p->next;
			RotatePosition = RotatePosition + 35;
		glPopMatrix();
	}

	return TRUE;
}

int CTaskManager::ProcessHit(int ObjectName)	// Process a hit on one of our objects
{
	nodeptr p = start;
	
	while (p != NULL && ObjectName != p->ObjectName)
		p = p->next;
		
	if (p != NULL)	// Object found: Move this window to front
	{
		if (IsIconic(p->hWnd)) ShowWindow(p->hWnd, SW_RESTORE);
		BringWindowToTop(p->hWnd);
		return TRUE;
	} else {
		return FALSE;
	}
}

int CTaskManager::UpdateTask(HWND hWnd)	// Make a new task if there isn't already one with the same hWnd, else, if there is, re-create its texture.
{
	if (IsInList (hWnd))	// We already have it: Update the texture
	{
		nodeptr prev, cur;
		int found = FALSE;
		
		cur = start;
		
		while (cur != NULL && hWnd != cur->hWnd)
		{
			prev = cur;
			cur = cur->next;
		}
		
		if (cur != NULL)
		{
			cur->RebuildTexture();
		}
	
		return TRUE;
	} else {
		return (AddTask(hWnd));
	}
}

int CTaskManager::AddTask(HWND hWnd)	// Make a new task if there isn't already one with the same hWnd.
{
	if (IsInList (hWnd)) return FALSE;	// It's already there

	nodeptr n, prev, cur;
	
	n = new CTask(hWnd, NamesOffset + TaskCount);
	n->hWnd = hWnd;
	TaskCount++;
	
	if (start == NULL)
	{
		start = n;
		n->next = NULL;
	}
	else
	{
		cur = start;
		while ( cur != NULL && hWnd > cur->hWnd )
		{
			prev = cur;
			cur = cur->next;
		}
		
		if (cur == start)
		{
			n -> next = start;
			start = n;
		}
		else
		{
			prev->next = n;
			n -> next = cur;
		}
	}

	return TRUE;
}

int CTaskManager::RemoveTask(HWND hWnd)	// Remove task with specified hWnd.
{
	nodeptr prev, cur;
	int found = FALSE;
	
	cur = start;
	
	while (cur != NULL && hWnd != cur->hWnd)
	{
		prev = cur;
		cur = cur->next;
	}
	
	if (cur != NULL)
	{
		if (hWnd == cur->hWnd)
		{
			if ( cur == start )
				start = start->next;
			else
				prev->next = cur->next;
			
			delete cur;
			TaskCount--;
			found = TRUE;
		}
	}
	
	return found;
}

bool CTaskManager::IsInList(HWND hWnd)	// Check existence of specified node by hWnd
{
	nodeptr p = start;
	
	while (p != NULL && hWnd != p->hWnd)
		p = p->next;
	
	if (p != NULL)
		return (hWnd == p->hWnd);
	else
		return false;
}
