/*
 *	3DShell
 *	Texture.cpp - Implements the texture management class CTexture
 *	Iain C, DATEMARK
 */

// ********************************* INCLUDES **********************************************

#include "GeneralHeader.h"

// ********************************* CONSTRUCTORS/DESTRUCTORS **********************************************

CTexture::CTexture()
{
}

CTexture::~CTexture()
{
	Toast();
}

// ********************************* METHODS **********************************************
int CTexture::Toast()
{
	glDeleteTextures(1, &texture);
	return TRUE;
}

int CTexture::Transparency()
{
	// Enable transparency

	// Enable blending
	glEnable(GL_BLEND);

	glBlendFunc(
		// GL_SRC_COLOR,	GL_DST_COLOR        // opaque envmap texture
		// GL_ONE_MINUS_SRC_ALPHA,GL_SRC_ALPHA	// only original texture
		// GL_ONE_MINUS_SRC_COLOR,GL_DST_COLOR  // opaque target
		// GL_DST_COLOR,GL_ONE_MINUS_SRC_COLOR	// as good as it gets
		// GL_ONE, GL_ONE_MINUS_SRC_ALPHA	    // unreal env (fog)
		// GL_DST_COLOR, GL_SRC_COLOR		    // unreal lightmap
		GL_ONE, GL_ONE_MINUS_SRC_COLOR			// unreal translucent
		);

	return TRUE;
}

int CTexture::Lightmap()
{
	// Enable lightmapping

	// Enable blending
	glEnable(GL_BLEND);

	glBlendFunc(
		// GL_SRC_COLOR,	GL_DST_COLOR        // opaque envmap texture
		// GL_ONE_MINUS_SRC_ALPHA,GL_SRC_ALPHA	// only original texture
		// GL_ONE_MINUS_SRC_COLOR,GL_DST_COLOR  // opaque target
		// GL_DST_COLOR,GL_ONE_MINUS_SRC_COLOR	// as good as it gets
		// GL_ONE, GL_ONE_MINUS_SRC_ALPHA	    // unreal env (fog)
		// GL_ONE, GL_ONE_MINUS_SRC_COLOR		// unreal translucent
		GL_DST_COLOR, GL_SRC_COLOR				// unreal lightmap
		);

	return TRUE;
}

int CTexture::EnvironmentMapping()
{
	// Set texture state to environment mapping

	// Set up the spheric coordinate generation
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, 
		// GL_EYE_LINEAR	// completely flat projector type projection from eye.
		// GL_OBJECT_LINEAR // like a normal fixed texture
		GL_SPHERE_MAP       // mapping relative to eye.  center of texture points toward viewer.
		);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, 
		// GL_EYE_LINEAR	// completely flat projector type projection from eye.
		// GL_OBJECT_LINEAR // like a normal fixed texture
		GL_SPHERE_MAP       // mapping relative to eye.  center of texture points toward viewer.
		);

	// Turn on the coordinate generate features for environment mapping
	glEnable(GL_TEXTURE_GEN_S); glEnable(GL_TEXTURE_GEN_T);

	return TRUE;
}

int CTexture::SaveState()
{
	// State should be saved before applying any texture effetcs
	glPushAttrib(GL_TEXTURE_BIT | GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	return TRUE;
}

int CTexture::RestoreState()
{
	// Restore state	
	glPopAttrib();
	return TRUE;
}

int CTexture::Use()
{
	glEnable(GL_TEXTURE_2D);							// Enable textures
	glBindTexture(GL_TEXTURE_2D, texture);
	return TRUE;
}

int CTexture::EndUse()
{
	glDisable(GL_TEXTURE_2D);							// Disable textures
	return TRUE;
}

AUX_RGBImageRec *CTexture::LoadBMP(char *Filename)		// Loads A Bitmap Image
{
	FILE *File=NULL;									// File Handle

	if (!Filename)										// Make Sure A Filename Was Given
	{
		return NULL;									// If Not Return NULL
	}

	File=fopen(Filename,"r");							// Check To See If The File Exists

	if (File)											// Does The File Exist?
	{
		fclose(File);									// Close The Handle
		return auxDIBImageLoad(Filename);				// Load The Bitmap And Return A Pointer
	}

	return NULL;										// If Load Failed Return NULL
}

int CTexture::CreateText(char *TextToDisplay)			// Create a text texture thing
{
	Toast();
	return TRUE;
}

int CTexture::CreateFromFile(char *Filename, int mode)			// Load Bitmaps And Convert To Textures
{
	Toast();
	//Terminate the filename properly
	char newFilename[64];
	unsigned long int fnLength = strlen(Filename);
	memcpy(newFilename, Filename, fnLength);
	newFilename[fnLength]='\0';
	Filename=newFilename;

	int Status=FALSE;									// Status Indicator

	AUX_RGBImageRec *TextureImage[1];					// Create Storage Space For The raw piccy

	memset(TextureImage,0,sizeof(void *)*1);           	// Set The Pointer To NULL

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP(Filename))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture);					// Create The Texture

		// Texture Generation Using Data From The Bitmap
		glBindTexture(GL_TEXTURE_2D, texture);

		switch (mode)	// Which texture creation mode to use
		{
			case 0:		// Nearest filtered texture
			{
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
				glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
			}

			case 1:		// Linear filtered texture
			{
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
				glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
			}

			case 2:		// Mipmapped texture
			{
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Close-up
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_NEAREST);	// Faraway
				gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
			}

			case 3:		// Anisotropically filtered texture
			{
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);	// Close-up
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_NEAREST);	// Faraway
				gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
			}
		}
	}

	if (TextureImage[0])									// If Texture Exists
	{
		if (TextureImage[0]->data)							// If Texture Image Exists
		{
			free(TextureImage[0]->data);					// Free The Texture Image Memory
		}

		free(TextureImage[0]);								// Free The Image Structure
	}

	return Status;										// Return The Status
}

int CTexture::CreateFromWindow(HWND hwnd)
{
	Toast();

//	BringWindowToTop(hwnd);

	HDC hdcScreen = GetWindowDC(hwnd);									// Screen DC
	HDC hdcCompatible = CreateCompatibleDC(hdcScreen);					// Memory DC
	HBITMAP hBitmap = CreateCompatibleBitmap(hdcScreen, 400, 400); 
	SelectObject(hdcCompatible, hBitmap);
	BitBlt(hdcCompatible, 0, 0, 400, 400, hdcScreen, 0,0, SRCCOPY);

	long bitmapData[400 * 400];

	int count = 0;
	for (int q = 0; q < 400; q++)
	{
		for (int w = 0; w < 400; w++)
		{
			bitmapData[count] = GetPixel(hdcCompatible, q, w);
			count++;
		}
	}

	glGenTextures(1, &texture);									// Generate name for and bind texture.
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, 400, 400, GL_BGRA_EXT, GL_UNSIGNED_BYTE, bitmapData);
/*
	RECT WRect;						// To get dimensions of window
	GetWindowRect(hwnd, &WRect);	// Get window dimensions
	int WindowWidth=(int)WRect.right - WRect.left;
	int WindowHeight=(int)WRect.bottom - WRect.top;

	BringWindowToTop(hwnd);

	HDC hdcScreen = GetWindowDC(hwnd);									// Screen DC
	HDC hdcCompatible = CreateCompatibleDC(hdcScreen);					// Memory DC
	HBITMAP hBitmap = CreateCompatibleBitmap(hdcScreen, WindowWidth, WindowHeight); 
	SelectObject(hdcCompatible, hBitmap);
	BitBlt(hdcCompatible, 0, 0, WindowWidth, WindowHeight, hdcScreen, 0,0, SRCCOPY);
	
//	int width = 3 * WindowWidth;
//	width = (width + 3) & ~3;
//	int ArraySize = width * WindowHeight;

	int ArraySize = WindowWidth * WindowHeight;
	long *bitmapData;
	bitmapData = (long *)calloc(ArraySize, sizeof(long));
		
	int count = 0;
	for (int q = 0; q < WindowWidth; q++)
	{
		for (int w = 0; w < WindowHeight; w++)
		{
			bitmapData[count] = GetPixel(hdcCompatible, q, w);
			count++;
		}
	}
	glGenTextures(1, &texture);									// Generate name for and bind texture.
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, WindowWidth, WindowHeight, GL_BGRA_EXT, GL_UNSIGNED_BYTE, bitmapData);
*/
    return TRUE;
}