/*
 *	3DShell
 *	Texture.h - Header file for class CTexture - a multi-purpose texture object
 *
 *  CreateFromFile modes:
 *		Mode 0 = Nearest filtered texture
 *		Mode 1 = Linear filtered texture
 *		Mode 2 = Mipmapped texture
 *		Mode 3 = Anisotropically filtured mipmapped texture
 *
 *	Iain C, DATEMARK
 */

class CTexture
{
public:
	// Constructor & destructor
	CTexture();
	~CTexture();

	// Creation Functions
	int	Toast();											// Wipe current texture: They can be re-used!
	int	CreateFromFile(char *Filename, int mode);			// Create a texture from a bitmap file with given mode (see above)

	int	CreateText(char *TextToDisplay);					// Create a texture from a text string
	int	CreateFromWindow(HWND hwnd);						// Create a texture from a window

	// Use Functions
	int	Use();												// Lock current texture for use with subsequent Texel values
	int	EndUse();											// Stop using the texture (disables textures, use before trying to do something with a colour just)

	// FX Functions
	int	RestoreState();										// Pop the GL attribute stack
	int	SaveState();										// Push the GL attribute stack
	int	Lightmap();											// Enable lightmap
	int	Transparency();										// Enable transparency
	int	EnvironmentMapping();								// Enable environment mapping

private:
	AUX_RGBImageRec *LoadBMP(char *Filename);				// Load a BMP
	PBITMAPINFO		CreateBitmapInfoStruct(HBITMAP hBmp);	// Create a BITMAPINFO structure
	GLuint			texture;								// Storage For the texture
	int				ScaleDIB(HWND hWnd);					// Scale a DIB
	long *ConvertRGB(void *originalBits, int PixWidth, int PixHeight);
	GLint			scale_down_to_power_of_2(LONG x);		// Little maths function, does exactly what it says on the tin
};
