/*
 *	3DShell
 *	Wharf.cpp - Implements the Wharf
 *	Iain C, DATEMARK
 */

// ********************************* INCLUDES **********************************************

#include "GeneralHeader.h"

// ********************************* CONSTRUCTORS/DESTRUCTORS **********************************************

CWharf::CWharf()
{
	float XStep = 3.4641;
	float YStep = 6.0f;

	// Using pre-calc values for tile positions
	TilePositionX[0]=XStep * 0;
	TilePositionY[0]=YStep * 0;

	TilePositionX[1]=-XStep * 1;
	TilePositionY[1]=YStep * 1;

	TilePositionX[2]=XStep * 1;
	TilePositionY[2]=YStep * 1;

	TilePositionX[3]=XStep * 2;
	TilePositionY[3]=YStep * 0;

	TilePositionX[4]=XStep * 1;
	TilePositionY[4]=-YStep * 1;
	
	TilePositionX[5]=-XStep * 1;
	TilePositionY[5]=-YStep * 1;
	
	TilePositionX[6]=-XStep * 2;
	TilePositionY[6]=YStep * 0;
	
	TilePositionX[7]=XStep * 0;
	TilePositionY[7]=YStep * 2;
	
	TilePositionX[8]=XStep * 2;
	TilePositionY[8]=YStep * 2;
	
	TilePositionX[9]=XStep * 3;
	TilePositionY[9]=YStep * 1;
	
	TilePositionX[10]=XStep * 4;
	TilePositionY[10]=YStep * 0;
	
	TilePositionX[11]=XStep * 3;
	TilePositionY[11]=-YStep * 1;
	
	TilePositionX[12]=XStep * 2;
	TilePositionY[12]=-YStep * 2;
	
	TilePositionX[13]=XStep * 0;
	TilePositionY[13]=-YStep * 2;
	
	TilePositionX[14]=-XStep * 2;
	TilePositionY[14]=-YStep * 2;
	
	TilePositionX[15]=-XStep * 3;;
	TilePositionY[15]=-YStep * 1;
	
	TilePositionX[16]=-XStep * 4;
	TilePositionY[16]=YStep * 0;
	
	TilePositionX[17]=-XStep * 3;
	TilePositionY[17]=YStep * 1;
	
	TilePositionX[18]=-XStep * 2;
	TilePositionY[18]=YStep * 2;
}

CWharf::~CWharf()
{
}

// ********************************* METHODS **********************************************

int CWharf::PostGLInit(int m_NamesOffset)	// Construct display list for wharf, set base name
{
	NamesOffset = m_NamesOffset;

	// Build display list for Wharf
	glNewList(1,GL_COMPILE);
		for (int i=0; i<19; i++)
		{
			glPushMatrix();
				glTranslatef(TilePositionX[i], TilePositionY[i], 0.0f);
				DrawHex();
			glPopMatrix();
		}
	glEndList();

	return TRUE;
}

int CWharf::Draw()	// Draw the Wharf. Wharf is a 4x4 grid (to allow for nice texture resolution) 40x40 in the z-plane. This actually leaves origin where it found it.
{
	glLoadName(NamesOffset+1);
	glCallList(1);							// Draw the Wharf
	return TRUE;
}

int CWharf::ProcessHit(int ObjectName)	// Process a hit on one of our objects
{
	//MessageBox (NULL, "Wharf got a hit!", "Hits!", MB_OK);
	return TRUE;
}

void CWharf::DrawHex()
{
	float x,y,angle;				// Calculated positions
	float diameter = 3.8f;			// Diameter of the head
	float height = 0.3f;			// Height
	float normal[3],corners[4][3];	// Storage of vertices and normals
	float step = (3.1415f/3.0f);	// step = 1/6th of a circle = hexagon

	// Set material color for top of segment
	glColor3f(1.0f, 1.0f, 1.0f);

	// Clockwise polygons face out, set for fans
	glFrontFace(GL_CW);

	// Begin a new triangle fan to cover the top
	glBegin(GL_TRIANGLE_FAN);

        // All the normals for the top of the bolt point straight up
        // the z axis.
        glNormal3f(0.0f, 0.0f, 1.0f);

        // Center of fan is at the origin
        glVertex3f(0.0f, 0.0f, 0.0f);

        // Divide the circle up into 6 sections and start dropping
        // points to specify the fan
        for(angle = 0.0f; angle < (2.0f*3.1415f); angle += step)
		{
			// Calculate x and y position of the next vertex
			x = diameter*(float)sin(angle);
			y = diameter*(float)cos(angle);

			// Specify the next vertex for the triangle fan
			glVertex3f(x, y, 0.0f);
        }

        // Last vertex closes the fan
        glVertex3f(0.0f, diameter, 0.0f);

	// Done drawing the fan that covers the top
	glEnd();

	// Build the sides out of triangles (two each). Each face
	// will consist of two triangles arranged to form a 
	// quadrilateral
	glBegin(GL_TRIANGLES);
	glColor3f(1.0f, 0.2f, 1.0f);

	// Go around and draw the sides
	for(angle = 0.0f; angle < (2.0f*3.1415f); angle += step)
	{
		// Calculate x and y position of the next hex point
		x = diameter*(float)sin(angle);
		y = diameter*(float)cos(angle);

		// start at bottom of head
		corners[0][0] = x;
		corners[0][1] = y;
		corners[0][2] = -height;

		// extrude to top of head
		corners[1][0] = x;
		corners[1][1] = y;
		corners[1][2] = 0.0f;

		// Calculate the next hex point
		x = diameter*(float)sin(angle+step);
		y = diameter*(float)cos(angle+step);

		// Make sure we aren't done before proceeding
		if(angle+step < 3.1415*2.0)
		{
			// If we are done, just close the fan at a
			// known coordinate.
			corners[2][0] = x;
			corners[2][1] = y;
			corners[2][2] = 0.0f;

			corners[3][0] = x;
			corners[3][1] = y;
			corners[3][2] = -height;
		} else {
			// We aren't done, the points at the top and bottom
			// of the head.
			corners[2][0] = 0.0f;
			corners[2][1] = diameter;
			corners[2][2] = 0.0f;

			corners[3][0] = 0.0f;
			corners[3][1] = diameter;
			corners[3][2] = -height;
		}

		// The normal vectors for the entire face will
		// all point the same direction
//		calcNormal(corners, normal);
		glNormal3fv(normal);

		// Specify each triangle separately to lie next
		// to each other.
		glVertex3fv(corners[0]);
		glVertex3fv(corners[1]);
		glVertex3fv(corners[2]);

		glVertex3fv(corners[0]);
		glVertex3fv(corners[2]);
		glVertex3fv(corners[3]);
	}

	glEnd();
}
